from .tex_config import TexConfig, DefaultConfig
from . import markdown as xwm
import re


class TexParser:
    def __init__(self):
        self.document = xwm.Document()
        self.in_code_block = False
        self.current_code_block = None
        self.current_list = None

    def parse(self, markdown_text):
        lines = markdown_text.splitlines()
        for line in lines:
            self._parse_line(line)

    def _preprocess_line(self, line):
        # remove leading and trailing whitespaces
        line = line.strip()
        # remove trailing comments
        line = re.sub(r'<!--.*-->', '', line)
        return line

    def _parse_line(self, line):

        # Codeblock
        if line.startswith("```"):
            if self.in_code_block:
                # close codeblock
                self.document.add_component(self.current_code_block)
                self.in_code_block = False
                self.current_code_block = None
            else:
                # open codeblock
                pattern = r"```(\w+)"
                match = re.match(pattern, line)
                if match:
                    language = match.group(1)
                    self.current_code_block = xwm.CodeBlock(lang=language)
                else:
                    self.current_code_block = xwm.CodeBlock()
                self.in_code_block = True
            return

        if self.in_code_block:
            # append code to codeblock
            self.current_code_block.add_code(line)
            return

        # remove leading and trailing whitespaces
        line = self._preprocess_line(line)

        # unordered list
        if re.match(r"^[-+*]\s", line):
            if self.current_list is None or not isinstance(self.current_list, xwm.UnorderedList):
                # start new unordered list
                self.current_list = xwm.UnorderedList()
                self.document.add_component(self.current_list)
            self._parse_list_item(line, self.current_list)
            return

        # ordered list
        if re.match(r"^\d+\.\s", line):
            if self.current_list is None or not isinstance(self.current_list, xwm.OrderedList):
                # start new ordered list
                self.current_list = xwm.OrderedList()
                self.document.add_component(self.current_list)
            self._parse_list_item(line, self.current_list)
            return
        
        self.current_list = None
        
        # heading
        pattern = r'^(#+)\s+(.*)'
        match = re.match(pattern, line)
        if match:
            level = len(match.group(1))
            title = self._parse_content(match.group(2))
            self.document.add_component(xwm.Heading(title=title, level=level))
            return
        
        # image
        pattern = r'!\[([^\]]+)\]\(([^)]+)\)'
        match = re.match(pattern, line)
        if match:
            caption = self._parse_content(match.group(1))
            path = match.group(2)
            self.document.add_component(xwm.Image(path=path, caption=caption))
            return
        
        # paragraph
        content = self._parse_content(line)     
        if len(content.components) > 0:
            self.document.add_component(xwm.Paragraph(content))

    def _parse_content(self, line):
        # 正则表达式匹配内联代码和公式
        pattern = r'(\*\*[^*]+\*\*|\*[^*]+\*|`[^`]+`|\$[^$]+\$|[^`$*]+)'
        matches = re.findall(pattern, line)

        content = xwm.Content()

        for match in matches:
            if match.startswith('`') and match.endswith('`'):
                # 内联代码
                content.add_component(xwm.InlineCode(match[1:-1]))
            elif match.startswith('$') and match.endswith('$'):
                # 数学公式
                content.add_component(xwm.InlineFormula(match[1:-1]))
            elif match.startswith('**') and match.endswith('**'):
                content.add_component(xwm.InlineBold(match[2:-2]))
            elif match.startswith('*') and match.endswith('*'):
                content.add_component(xwm.InlineItalic(match[1:-1]))
            else:
                # 普通文本
                content.add_component(xwm.Text(match))
                
        return content

    def _parse_list_item(self, line, list_obj):
        item = xwm.ListItem()
        content = re.sub(r"^[-+*]\s|\d+\.\s", "", line).strip()
        item.add_component(self._parse_content(content))
        list_obj.add_item(item)

    @property
    def doc(self):
        return self.document


import yaml
import importlib.resources as pkg_resources
class TexConfig:
    def __init__(self, config_file = None):
        self.config = {}
        if config_file is not None:
            self.load(config_file)
        else:
            self.load_default()
            
    def load(self, config_file):
        with open(config_file, 'r') as f:
            self.config = yaml.load(f, Loader=yaml.FullLoader)
    
    def load_default(self):
        with pkg_resources.open_text('texweaver', 'default.yaml') as f:
            self.config = yaml.load(f, Loader=yaml.FullLoader)
    
    def apply(self, key, **kwargs):
        if key in self.config:
            return self.config[key].format(**kwargs)
        elif 'content' in kwargs:
            return kwargs['content']
        else:
            return ''
        

DefaultConfig = TexConfig()


from texweaver import TexParser
from texweaver import DefaultConfig
def test_parser():
    with open('test/test.md', 'r') as f:
        src = f.read()
    
    parser = TexParser()
    parser.parse(src)
    mdoc = parser.doc
    latex = mdoc.to_latex(DefaultConfig)
    print(latex)
    
if __name__ == '__main__':
    test_parser()


from texweaver import TexParser
def test_parser():
    with open('test/test.md', 'r') as f:
        src = f.read()
    parser = TexParser()
    parser.parse(src)
    print(parser.doc.to_json())
    
if __name__ == '__main__':
    test_parser()


from .tex_config import TexConfig
import json

def preprocess_text(text):
    # replace underscores with \_
    text = text.replace('_', r'\_')
    return text

class Document:
    def __init__(self):
        self.components = []
        
    def add_component(self, component):
        self.components.append(component)    
    
    def to_latex(self, config : TexConfig):
        return '\n'.join([c.to_latex(config) for c in self.components])
    
    def to_json(self):
        obj = {
            'type': 'document',
            'components': [c.to_json() for c in self.components]
        }
        json_str = json.dumps(obj, indent=4)
        return json_str
        

class Content:
    def __init__(self):
        self.components = []
    
    def add_component(self, component):
        self.components.append(component)
        
    def to_latex(self, config : TexConfig):
        return ''.join([c.to_latex(config) for c in self.components])
    
    def to_json(self):
        return {
            'type': 'content',
            'components': [c.to_json() for c in self.components]
        }        

class Text:
    def __init__(self, text: str):
        self.text = preprocess_text(text)
        
    def to_latex(self, config : TexConfig):
        return config.apply('text', content=self.text)
    
    def to_json(self):
        return {
            'type': 'text',
            'text': self.text
        }
    
class InlineBold:
    def __init__(self, text: str):
        self.text = preprocess_text(text)
        
    def to_latex(self, config : TexConfig):
        return config.apply('bold', content=self.text)
    
    def to_json(self):
        return {
            'type': 'inline_bold',
            'text': self.text
        }
    
class InlineItalic:
    def __init__(self, text: str):
        self.text = preprocess_text(text)
        
    def to_latex(self, config : TexConfig):
        return config.apply('italic', content=self.text)
    
    def to_json(self):
        return {
            'type': 'inline_italic',
            'text': self.text
        }
    
class InlineCode:
    def __init__(self, text: str):
        self.text = preprocess_text(text)
        
    def to_latex(self, config : TexConfig):
        return config.apply('inline_code', content=self.text)
    
    def to_json(self):
        return {
            'type': 'inline_code',
            'text': self.text
        }
    
class InlineFormula:
    def __init__(self, text: str):
        self.text = text
        
    def to_latex(self, config : TexConfig):
        return config.apply('inline_formula', content=self.text)
    
    def to_json(self):
        return {
            'type': 'inline_formula',
            'text': self.text
        }
        
class Paragraph:
    def __init__(self, content: Content):
        self.content = content
        
    def to_latex(self, config : TexConfig):
        return config.apply('paragraph', content=self.content.to_latex(config))
    
    def to_json(self):
        return {
            'type': 'paragraph',
            'content': self.content.to_json()
        }
        
class FormulaBlock:
    def __init__(self, text):
        self.text = text
        
    def to_latex(self, config : TexConfig):
        return config.apply('formula_block', content=self.text)
    
    def to_json(self):
        return {
            'type': 'formula_block',
            'text': self.text
        }
    
class CodeBlock:
    def __init__(self, lang = None):
        self.lang = lang if lang else 'text'
        self.code = []
        
    def add_code(self, code):
        self.code.append(code)
        
    def to_latex(self, config : TexConfig):
        return config.apply('code_block', code='\n'.join(self.code), lang=self.lang)
    def to_json(self):
        return {
            'type': 'code_block',
            'code': self.code,
            'lang': self.lang
        }
    
class Image:
    def __init__(self, path: str, caption: Content):
        self.path = path
        self.caption = caption
        
    def to_latex(self, config : TexConfig):
        return config.apply('image', src=self.path, alt=self.caption.to_latex(config))
    
    def to_json(self):
        return {
            'type': 'image',
            'path': self.path,
            'caption': self.caption.to_json()
        }
    
class Heading:
    def __init__(self, title: Content, level):
        self.title = title
        self.level = level
        
    def to_latex(self, config : TexConfig):
        content = self.title.to_latex(config)
        if self.level == 1:
            return config.apply('heading1', content=content)
        elif self.level == 2:
            return config.apply('heading2', content=content)
        elif self.level == 3:
            return config.apply('heading3', content=content)
        elif self.level == 4:
            return config.apply('heading4', content=content)
        elif self.level == 5:
            return config.apply('heading5', content=content)
        else:
            return config.apply('bold', content=content)
        
    def to_json(self):
        return {
            'type': 'heading',
            'title': self.title.to_json(),
            'level': self.level,
        }
    
class OrderedList:
    def __init__(self):
        self.items = []
        
    def add_item(self, item):
        self.items.append(item)
        
    def to_latex(self, config : TexConfig):
        return config.apply('ordered_list', items='\n'.join([i.to_latex(config) for i in self.items]))
    
    def to_json(self):
        return {
            'type': 'ordered_list',
            'items': [i.to_json() for i in self.items]
        }
    
class UnorderedList:
    
    def __init__(self):
        self.items = []
        
    def add_item(self, item):
        self.items.append(item)
        
    def to_latex(self, config : TexConfig):
        return config.apply('unordered_list', items='\n'.join([i.to_latex(config) for i in self.items]))
    
    def to_json(self):
        return {
            'type': 'unordered_list',
            'items': [i.to_json() for i in self.items]
        }
    
class ListItem:
    def __init__(self):
        self.components = []
        
    def add_component(self, component):
        self.components.append(component)
        
    def to_latex(self, config : TexConfig):
        return config.apply('list_item', content=''.join([c.to_latex(config) for c in self.components]))
    
    def to_json(self):
        return {
            'type': 'list_item',
            'components': [c.to_json() for c in self.components]
        }



from setuptools import setup, find_packages

setup(
    name='texweaver',
    version='0.3.1',
    description='A simple converter from Markdown to LaTeX',
    author='Eric025',
    author_email='erix025@outlook.com',
    url='https://github.com/erix025/texweaver',
    packages=find_packages(),
    package_data={
        'texweaver': ['default.yaml'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'texweaver = texweaver.entrypoint:main',
        ],
    },
    install_requires=[
        'argparse'
    ],
)


import argparse
from . import TexParser, DefaultConfig

def main():
    # Add your CLI logic here
    parser = argparse.ArgumentParser(description='Texweaver CLI')
    parser.add_argument('input_file', help='Path to input file')
    parser.add_argument('output_file', help='Path to output file')
    # Add more arguments as needed

    args = parser.parse_args()

    # Process the input file and generate the output file
    process_file(args.input_file, args.output_file)

def process_file(input_file, output_file):
    # Implement your file processing logic here
    parser = TexParser()
    with open(input_file, 'r', encoding='utf-8') as f:
        src = f.read()
        parser.parse(src)
    doc = parser.doc
    with open(output_file, 'w', encoding='utf-8') as f:
        f.write(doc.to_latex(DefaultConfig))

if __name__ == '__main__':
    main()